@@ -2,10 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2016 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 /* The stand-alone version just tests the algorithm. We have to drag
@@ -13,27 +11,25 @@ in the MD5 computation functions, without their own stand-alone main
 program. */
 
 #ifdef STAND_ALONE
-# define CRAM_STAND_ALONE
-# include "md5.c"
+#define CRAM_STAND_ALONE
+#include "md5.c"
 
 
 /* This is the normal, non-stand-alone case */
 
 #else
-# include "../exim.h"
-
-# ifdef AUTH_CRAM_MD5
-#  include "cram_md5.h"
+#include "../exim.h"
+#include "cram_md5.h"
 
 /* Options specific to the cram_md5 authentication mechanism. */
 
 optionlist auth_cram_md5_options[] = {
   { "client_name",        opt_stringptr,
-      OPT_OFF(auth_cram_md5_options_block, client_name) },
+      (void *)(offsetof(auth_cram_md5_options_block, client_name)) },
   { "client_secret",      opt_stringptr,
-      OPT_OFF(auth_cram_md5_options_block, client_secret) },
+      (void *)(offsetof(auth_cram_md5_options_block, client_secret)) },
   { "server_secret",      opt_stringptr,
-      OPT_OFF(auth_cram_md5_options_block, server_secret) }
+      (void *)(offsetof(auth_cram_md5_options_block, server_secret)) }
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -51,17 +47,6 @@ auth_cram_md5_options_block auth_cram_md5_option_defaults = {
 };
 
 
-#  ifdef MACRO_PREDEF
-
-/* Dummy values */
-void auth_cram_md5_init(driver_instance* ablock) {}
-int auth_cram_md5_server(auth_instance *ablock, uschar *data) {return 0;}
-int auth_cram_md5_client(auth_instance *ablock, void *sx, int timeout,
-    uschar *buffer, int buffsize) {return 0;}
-
-#  else	/*!MACRO_PREDEF*/
-
-
 /*************************************************
 *          Initialization entry point            *
 *************************************************/
@@ -71,28 +56,22 @@ enable consistency checks to be done, or anything else that needs
 to be set up. */
 
 void
-auth_cram_md5_init(driver_instance * a)
+auth_cram_md5_init(auth_instance *ablock)
 {
-auth_instance * ablock = (auth_instance *)a;
-auth_cram_md5_options_block * ob =
-  (auth_cram_md5_options_block *)(a->options_block);
-
-if (ob->server_secret)
-  ablock->server = TRUE;
-if (ob->client_secret)
+auth_cram_md5_options_block *ob =
+  (auth_cram_md5_options_block *)(ablock->options_block);
+if (ob->server_secret != NULL) ablock->server = TRUE;
+if (ob->client_secret != NULL)
   {
   ablock->client = TRUE;
-  if (!ob->client_name) ob->client_name = primary_hostname;
+  if (ob->client_name == NULL) ob->client_name = primary_hostname;
   }
 }
 
-#  endif	/*!MACRO_PREDEF*/
-# endif		/*AUTH_CRAM_MD5*/
-#endif		/*!STAND_ALONE*/
+#endif  /* STAND_ALONE */
 
 
 
-#ifndef MACRO_PREDEF
 /*************************************************
 *      Perform the CRAM-MD5 algorithm            *
 *************************************************/
@@ -117,6 +96,7 @@ static void
 compute_cram_md5(uschar *secret, uschar *challenge, uschar *digestptr)
 {
 md5 base;
+int i;
 int len = Ustrlen(secret);
 uschar isecret[64];
 uschar osecret[64];
@@ -128,8 +108,8 @@ and use that. */
 if (len > 64)
   {
   md5_start(&base);
-  md5_end(&base, US secret, len, md5secret);
-  secret = US md5secret;
+  md5_end(&base, (uschar *)secret, len, md5secret);
+  secret = (uschar *)md5secret;
   len = 16;
   }
 
@@ -140,7 +120,7 @@ memcpy(isecret, secret, len);
 memset(isecret+len, 0, 64-len);
 memcpy(osecret, isecret, 64);
 
-for (int i = 0; i < 64; i++)
+for (i = 0; i < 64; i++)
   {
   isecret[i] ^= 0x36;
   osecret[i] ^= 0x5c;
@@ -150,7 +130,7 @@ for (int i = 0; i < 64; i++)
 
 md5_start(&base);
 md5_mid(&base, isecret);
-md5_end(&base, US challenge, Ustrlen(challenge), md5secret);
+md5_end(&base, (uschar *)challenge, Ustrlen(challenge), md5secret);
 
 /* Compute the outer MD5 digest */
 
@@ -160,8 +140,7 @@ md5_end(&base, md5secret, 16, digestptr);
 }
 
 
-# ifndef STAND_ALONE
-#  ifdef AUTH_CRAM_MD5
+#ifndef STAND_ALONE
 
 /*************************************************
 *             Server entry point                 *
@@ -170,29 +149,30 @@ md5_end(&base, md5secret, 16, digestptr);
 /* For interface, see auths/README */
 
 int
-auth_cram_md5_server(auth_instance * ablock, uschar * data)
+auth_cram_md5_server(auth_instance *ablock, uschar *data)
 {
-auth_cram_md5_options_block * ob = ablock->drinst.options_block;
-uschar * challenge = string_sprintf("<%d.%ld@%s>", getpid(),
+auth_cram_md5_options_block *ob =
+  (auth_cram_md5_options_block *)(ablock->options_block);
+uschar *challenge = string_sprintf("<%d.%ld@%s>", getpid(),
     (long int) time(NULL), primary_hostname);
-uschar * clear, * secret;
+uschar *clear, *secret;
 uschar digest[16];
 int i, rc, len;
 
 /* If we are running in the test harness, always send the same challenge,
 an example string taken from the RFC. */
 
-if (f.running_in_test_harness)
+if (running_in_test_harness)
   challenge = US"<1896.697170952@postoffice.reston.mci.net>";
 
 /* No data should have been sent with the AUTH command */
 
-if (*data) return UNEXPECTED;
+if (*data != 0) return UNEXPECTED;
 
 /* Send the challenge, read the return */
 
 if ((rc = auth_get_data(&data, challenge, Ustrlen(challenge))) != OK) return rc;
-if ((len = b64decode(data, &clear, GET_TAINTED)) < 0) return BAD64;
+if ((len = b64decode(data, &clear)) < 0) return BAD64;
 
 /* The return consists of a user name, space-separated from the CRAM-MD5
 digest, expressed in hex. Extract the user name and put it in $auth1 and $1.
@@ -200,7 +180,7 @@ The former is now the preferred variable; the latter is the original one. Then
 check that the remaining length is 32. */
 
 auth_vars[0] = expand_nstring[1] = clear;
-Uskip_nonwhite(&clear);
+while (*clear != 0 && !isspace(*clear)) clear++;
 if (!isspace(*clear)) return FAIL;
 *clear++ = 0;
 
@@ -219,7 +199,7 @@ the given name. */
 
 if (secret == NULL)
   {
-  if (f.expand_string_forcedfail) return FAIL;
+  if (expand_string_forcedfail) return FAIL;
   auth_defer_msg = expand_string_message;
   return DEFER;
   }
@@ -230,10 +210,13 @@ compute_cram_md5(secret, challenge, digest);
 
 HDEBUG(D_auth)
   {
+  uschar buff[64];
   debug_printf("CRAM-MD5: user name = %s\n", auth_vars[0]);
   debug_printf("          challenge = %s\n", challenge);
   debug_printf("          received  = %s\n", clear);
-  debug_printf("          digest    = %.16H\n", digest);
+  Ustrcpy(buff,"          digest    = ");
+  for (i = 0; i < 16; i++) sprintf(CS buff+22+2*i, "%02x", digest[i]);
+  debug_printf("%.54s\n", buff);
   }
 
 /* We now have to compare the digest, which is 16 bytes in binary, with the
@@ -263,12 +246,14 @@ return auth_check_serv_cond(ablock);
 int
 auth_cram_md5_client(
   auth_instance *ablock,                 /* authenticator block */
-  void * sx,				 /* smtp connextion */
+  smtp_inblock *inblock,                 /* input connection */
+  smtp_outblock *outblock,               /* output connection */
   int timeout,                           /* command timeout */
   uschar *buffer,                        /* for reading response */
   int buffsize)                          /* size of buffer */
 {
-auth_cram_md5_options_block * ob = ablock->drinst.options_block;
+auth_cram_md5_options_block *ob =
+  (auth_cram_md5_options_block *)(ablock->options_block);
 uschar *secret = expand_string(ob->client_secret);
 uschar *name = expand_string(ob->client_name);
 uschar *challenge, *p;
@@ -280,27 +265,27 @@ or ERROR, as appropriate. */
 
 if (!secret || !name)
   {
-  if (f.expand_string_forcedfail)
+  if (expand_string_forcedfail)
     {
     *buffer = 0;           /* No message */
     return CANCELLED;
     }
-  string_format(buffer, buffsize, "expansion of %q failed in "
+  string_format(buffer, buffsize, "expansion of \"%s\" failed in "
     "%s authenticator: %s",
     !secret ? ob->client_secret : ob->client_name,
-    ablock->drinst.name, expand_string_message);
+    ablock->name, expand_string_message);
   return ERROR;
   }
 
 /* Initiate the authentication exchange and read the challenge, which arrives
 in base 64. */
 
-if (smtp_write_command(sx, SCMD_FLUSH, "AUTH %s\r\n", ablock->public_name) < 0)
+if (smtp_write_command(outblock, FALSE, "AUTH %s\r\n", ablock->public_name) < 0)
   return FAIL_SEND;
-if (!smtp_read_response(sx, buffer, buffsize, '3', timeout))
+if (!smtp_read_response(inblock, buffer, buffsize, '3', timeout))
   return FAIL;
 
-if (b64decode(buffer + 4, &challenge, buffer + 4) < 0)
+if (b64decode(buffer + 4, &challenge) < 0)
   {
   string_format(buffer, buffsize, "bad base 64 string in challenge: %s",
     big_buffer + 4);
@@ -311,58 +296,39 @@ if (b64decode(buffer + 4, &challenge, buffer + 4) < 0)
 
 compute_cram_md5(secret, challenge, digest);
 
-/* Create the response from the user name plus the CRAM-MD5 digest.
-Send it, in base 64, and check the result. */
+/* Create the response from the user name plus the CRAM-MD5 digest */
+
+string_format(big_buffer, big_buffer_size - 36, "%s", name);
+for (p = big_buffer; *p; ) p++;
+*p++ = ' ';
+
+for (i = 0; i < 16; i++)
   {
-  int len;
-  p = string_sprintf("%s %.16H%n", name, digest, &len);
-  if (smtp_write_command(sx, SCMD_FLUSH, "%s\r\n", b64encode(p, len)) < 0)
-    return FAIL_SEND;
+  sprintf(CS p, "%02x", digest[i]);
+  p += 2;
   }
 
-return smtp_read_response(sx, US buffer, buffsize, '2', timeout)
+/* Send the response, in base 64, and check the result. The response is
+in big_buffer, but b64encode() returns its result in working store,
+so calling smtp_write_command(), which uses big_buffer, is OK. */
+
+buffer[0] = 0;
+if (smtp_write_command(outblock, FALSE, "%s\r\n", b64encode(big_buffer,
+  p - big_buffer)) < 0) return FAIL_SEND;
+
+return smtp_read_response(inblock, (uschar *)buffer, buffsize, '2', timeout)
   ? OK : FAIL;
 }
+#endif  /* STAND_ALONE */
 
 
-# ifdef DYNLOOKUP
-#  define cram_md5_auth_info _auth_info
-# endif
-#  endif  /*AUTH_CRAM_MD5*/
-# endif  /*!STAND_ALONE*/
-#endif	/*!MACRO_PREDEF*/
-
-#ifndef STAND_ALONE
-# ifdef AUTH_CRAM_MD5
-
-auth_info cram_md5_auth_info = {
-.drinfo = {
-  .driver_name =	US"cram_md5",                   /* lookup name */
-  .options =		auth_cram_md5_options,
-  .options_count =	&auth_cram_md5_options_count,
-  .options_block =	&auth_cram_md5_option_defaults,
-  .options_len =	sizeof(auth_cram_md5_options_block),
-  .init =		auth_cram_md5_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		AUTH_MAGIC,
-# endif
-  },
-.servercode =		auth_cram_md5_server,
-.clientcode =		auth_cram_md5_client,
-.version_report =	NULL,
-.macros_create =	NULL,
-};
-
-# endif  /*AUTH_CRAM_MD5*/
-#endif  /*!STAND_ALONE*/
-
 /*************************************************
 **************************************************
 *             Stand-alone test program           *
 **************************************************
 *************************************************/
 
-# ifdef STAND_ALONE
+#ifdef STAND_ALONE
 
 int main(int argc, char **argv)
 {
@@ -379,6 +345,6 @@ printf("\n");
 return 0;
 }
 
-# endif	/*STAND_ALONE*/
+#endif
 
 /* End of cram_md5.c */
